
import torch.nn as nn


class CanberLoss(nn.Module):
    def __init__(self, reduction='batchmean'):
        super().__init__()
        assert reduction in ('batchmean',)

    def forward(self, input, target):
        canber = (input-target).abs()/(input+target)
        canber = canber.sum(dim=1).mean()
        return canber



import torch
from tqdm import tqdm

from .utils import check_loss, get_loaders, load_checkpoint, save_checkpoint


# Sample transform
# sample_transform = A.Compose([
#     A.Resize(224),
#     A.RandomRotation(30),
#     A.RandomCrop((224, 224)),
#     A.RandomHorizontalFlip(p=0.5),
#     A.ToTensor(),
#     A.Normalize(mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]),
# ])

# Hyper parameters
NUM_WORKERS = 2
PIN_MEMORY = True
MAX_EPOCH = 50
DEVICE = 'cuda' if torch.cuda.is_available() else 'cpu'
SMOOTH_FACTOR = 0.2


def train_fn(loader, model, optimizer, loss_fn, scaler, device):
    loop = tqdm(loader)
    smooth_loss = None

    for batch_idx, (data, targets) in enumerate(loop):
        data = data.to(device=device)
        targets = targets.to(device=device)

        # forward
        with torch.cuda.amp.autocast():
            predictions = model(data)
            loss = loss_fn(predictions, targets)

        # backward
        optimizer.zero_grad()
        scaler.scale(loss).backward()
        scaler.step(optimizer)
        scaler.update()

        # update tqdm loop
        loss = loss.item()
        if smooth_loss is None:
            smooth_loss = loss
        smooth_loss = SMOOTH_FACTOR * smooth_loss + (1-SMOOTH_FACTOR) * loss
        loop.set_postfix(loss=loss, smooth_loss=smooth_loss)


def loop_fn(model, optimizer, loss_fn, trainset, valset, batch_size, max_patience=10, checkpoint=None, model_name='my_model'):
    if checkpoint is not None:
        load_checkpoint(model, optimizer, checkpoint)

    train_loader, val_loader = get_loaders(trainset, valset, batch_size, NUM_WORKERS, PIN_MEMORY)
    min_loss = float('inf')
    scaler = torch.cuda.amp.GradScaler()

    for epoch in range(MAX_EPOCH):
        train_fn(train_loader, model, optimizer, loss_fn, scaler, DEVICE)
        loss = check_loss(val_loader, model, DEVICE, epoch)
        # early stop
        if loss < min_loss:
            save_checkpoint(model, optimizer, file_name=f'{model_name}.pth.tar')
            min_loss = loss
            patience = max_patience
        else:
            patience -= 1
            if patience == 0:
                break




import torch.nn as nn


class SpatialSoftmax2d(nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, x):
        shape = x.shape
        x = x.flatten(start_dim=2)
        x = x.softmax(dim=2)
        x = x.view(shape)
        return x



import os
from PIL import Image
from torch.utils.data import Dataset
import numpy as np
import json


class FlickrDataset(Dataset):

    def __init__(self, root, train=True, transforms=None, fold=0):
        super().__init__()

        with open(root + f'/checklist_fold_{fold}.json', 'r') as f:
            self.data = json.load(f)['train' if train else 'val']

        # read samples
        self.root = os.path.join(root, 'data/')
        self.transforms = transforms

    def __len__(self):
        return len(self.data)

    def __getitem__(self, index):
        image_name, label = self.data[index]
        image_path = os.path.join(self.root, image_name)
        image = Image.open(image_path).convert('RGB')
        label = np.array(label, dtype=np.float32)

        if self.transforms is not None:
            image = self.transforms(image)
        label = label / sum(label)

        return image, label


if __name__ == "__main__":
    model = FlickrDataset('/home/sh/Datasets/Flickr_LDL')
    print(model.__getitem__(1))



import torch.nn as nn


class ChebyshevLoss(nn.Module):
    def __init__(self, reduction='batchmean'):
        super().__init__()
        assert reduction in ('batchmean',)

    def forward(self, input, target):
        cheb = (input-target).abs().max(dim=1)[0]
        cheb = cheb.mean()
        return cheb



#!/usr/bin/env python
# -*- coding:utf-8 -*-


from setuptools import setup, find_packages

setup(
    name="torchmods",
    version="0.5.21",
    keywords=("cv", "pytorch", "auxiliary"),
    description="mods for torch & cv",
    long_description="experimental mods for pytorch & cv research",
    license="MIT Licence",

    url="https://github.com/klrc/torchmods",
    author="klrc",
    author_email="sh@mail.ecust.edu.com",

    packages=find_packages(),
    include_package_data=True,
    platforms=["all"],
    install_requires=[
        "torch",
        "matplotlib",
        "paramiko",
        "opencv_python",
        "requests",
        "numpy",
        "torchvision",
        "beautifulsoup4",
        "imageio",
        "Pillow",
        "pynvml",
    ]
)




import torch
import torch.nn as nn


class ClarkLoss(nn.Module):
    def __init__(self, reduction='batchmean'):
        super().__init__()
        assert reduction in ('batchmean',)

    def forward(self, input, target):
        clark = (input-target)/(input+target)
        clark = torch.norm(clark, dim=1).mean()
        return clark



import os
from torch.utils.data import Dataset
import cv2


class SALICONDataset(Dataset):

    def __init__(self, root, train=True, transforms=None):
        super().__init__()
        assert os.path.exists(root)
        src_dir = 'train' if train else 'val'
        src_dir = f'{root}/images/{src_dir}'
        self.data = [f'{src_dir}/{x}' for x in os.listdir(src_dir)]
        self.transform = transforms

    def __cvimg__(self, path, convert2rgb=True):
        """
            read image in RGB mode.
        """
        img_cv = cv2.imread(path)
        if convert2rgb:
            img_cv = cv2.cvtColor(img_cv, cv2.COLOR_BGR2RGB)
        return img_cv

    def __getitem__(self, index):
        """
            function for torch.util.data.Dataset class,
            returns image, (target_1, target_2, ..., target_n)

            Args:
                index:
        """
        img = self.__cvimg__(self.data[index])
        target = cv2.imread((self.data[index].replace('/images/', '/maps/').replace('.jpg', '.png')), 0)

        # for albumentations only!
        if self.transform:
            output = self.transform(image=img, mask=target)
            img = output['image']
            target = output['mask']

        return img, target

    def __len__(self):
        """
            function for torch.util.data.Dataset class.
        """
        return len(self.data)
